/* A simple tool to update the WR PTP core software from a Linux terminal. */

#include <stdio.h>
#include <unistd.h>
#include <arpa/inet.h>

#include "vfchdlib.h"
#include "wrc_syscon_regs.h"

uint32_t vfchd_wrcmem_read(struct vfchd_dev *dev, uint32_t offset)
{
	return vfchd_wrc_readl(dev, dev->wrc_mem_base + offset);
}


void vfchd_wrcmem_write(struct vfchd_dev *dev, uint32_t datum, uint32_t offset)
{
	vfchd_wrc_writel(dev, datum, dev->wrc_mem_base + offset);
}

int vfchd_wrc_loader (struct vfchd_dev *dev, const char *fname)
{

	int retval = 0;

	FILE *f = fopen(fname, "rb");
	if ( f == NULL )
	{
		fprintf( stderr, "(EE): Could not access file %s\n", fname );
		return -1;
	}

	fseek(f, 0, SEEK_END);

	long fsize = ftell(f);

	fseek(f, 0, SEEK_SET);

	uint32_t datum, readback;

	int i = 0;

	vfchd_wrc_disable(dev);

	for (i = 0; i < fsize; i += 4) {

		fread((struct vfchd_dev *)&datum, 4, 1, f);

		vfchd_wrcmem_write(dev, ntohl(datum), i);

		readback = vfchd_wrcmem_read(dev, i);

		if ( readback != ntohl(datum)) {
			fprintf(stderr,
				"(EE): error writing value 0x%.8x "
				"to address 0x%.8x. "
				"Read-back value was 0x%.8x\n",
				ntohl(datum), i, readback);
			retval = -1;
			break;
		}
	}

	fclose(f);

	vfchd_wrc_enable(dev);

	return retval;
}

int main(int argc, char **argv)
{
	struct vfchd_dev *dev = vfchd_open(argc, argv);

	if (!dev)
		return 1;

	if (optind >= argc) {
		fprintf(stderr, "(EE): expected binary name after options.\n");
		return 1;
	}

	if (vfchd_wrc_loader(dev, argv[optind]) < 0)
	{
		fprintf(stderr, "(EE): WRC loader failure.\n");
		return 1;
	}

	vfchd_close(dev);

	return 0;
}
